@@ -1,4 +1,4 @@
-# Copyright 2020 Oliver Smith
+# Copyright 2021 Oliver Smith
 # SPDX-License-Identifier: GPL-3.0-or-later
 import os
 import logging
@@ -6,6 +6,7 @@ import shlex
 
 import pmb.chroot
 import pmb.config
+import pmb.helpers.apk
 import pmb.helpers.pmaports
 import pmb.parse.apkindex
 import pmb.parse.arch
@@ -77,13 +78,10 @@ def check_min_version(args, suffix="native"):
 
     # Compare
     version_installed = installed(args, suffix)["apk-tools"]["version"]
-    version_min = pmb.config.apk_tools_static_min_version
-    if pmb.parse.version.compare(version_installed, version_min) == -1:
-        raise RuntimeError("You have an outdated version of the 'apk' package"
-                           " manager installed (your version: " + version_installed +
-                           ", expected at least: " + version_min + "). Delete"
-                           " your http cache and zap all chroots, then try again:"
-                           " 'pmbootstrap zap -hc'")
+    pmb.helpers.apk.check_outdated(
+        args, version_installed,
+        "Delete your http cache and zap all chroots, then try again:"
+        " 'pmbootstrap zap -hc'")
 
     # Mark this suffix as checked
     args.cache["apk_min_version_checked"].append(suffix)
@@ -231,10 +229,18 @@ def install(args, packages, suffix="native", build=True):
         commands = [["add", "-u", "--virtual", ".pmbootstrap"] + packages_todo,
                     ["add"] + packages,
                     ["del", ".pmbootstrap"]]
-    for command in commands:
+    for (i, command) in enumerate(commands):
         if args.offline:
             command = ["--no-network"] + command
-        pmb.chroot.root(args, ["apk", "--no-progress"] + command, suffix=suffix, disable_timeout=True)
+        if i == 0:
+            pmb.helpers.apk.apk_with_progress(args, ["apk"] + command,
+                                              chroot=True, suffix=suffix)
+        else:
+            # Virtual package related commands don't actually install or remove
+            # packages, but only mark the right ones as explicitly installed.
+            # They finish up almost instantly, so don't display a progress bar.
+            pmb.chroot.root(args, ["apk", "--no-progress"] + command,
+                            suffix=suffix)
 
 
 def installed(args, suffix="native"):
