@@ -1,4 +1,4 @@
-# Copyright 2020 Oliver Smith
+# Copyright 2021 Oliver Smith
 # SPDX-License-Identifier: GPL-3.0-or-later
 import os
 import logging
@@ -7,6 +7,7 @@ import tarfile
 import tempfile
 import stat
 
+import pmb.helpers.apk
 import pmb.helpers.run
 import pmb.config
 import pmb.config.load
@@ -152,16 +153,12 @@ def init(args):
                                             indexes=[apkindex])
     version = index_data["version"]
 
-    # Extract and verify the apk-tools-static version
-    version_min = pmb.config.apk_tools_static_min_version
-    apk_name = "apk-tools-static-" + version + ".apk"
-    if pmb.parse.version.compare(version, version_min) == -1:
-        raise RuntimeError("Your APKINDEX has an outdated version of"
-                           " apk-tools-static (your version: " + version +
-                           ", expected at least:" + version_min + "). Please" +
-                           " run 'pmbootstrap update'.")
+    # Verify the apk-tools-static version
+    pmb.helpers.apk.check_outdated(
+        args, version, "Run 'pmbootstrap update', then try again.")
 
     # Download, extract, verify apk-tools-static
+    apk_name = "apk-tools-static-" + version + ".apk"
     apk_static = download(args, apk_name)
     extract(args, version, apk_static)
 
@@ -169,4 +166,4 @@ def init(args):
 def run(args, parameters):
     if args.offline:
         parameters = ["--no-network"] + parameters
-    pmb.helpers.run.root(args, [args.work + "/apk.static"] + parameters)
+    pmb.helpers.apk.apk_with_progress(args, [args.work + "/apk.static"] + parameters, chroot=False)
