@@ -1,4 +1,4 @@
-# Copyright 2020 Oliver Smith
+# Copyright 2021 Oliver Smith
 # SPDX-License-Identifier: GPL-3.0-or-later
 import os
 import shutil
@@ -53,6 +53,7 @@ def root(args, cmd, suffix="native", working_dir="/", output="log",
 
     # Merge env with defaults into env_all
     env_all = {"CHARSET": "UTF-8",
+               "LANG": "UTF-8",
                "HISTFILE": "~/.ash_history",
                "HOME": "/root",
                "PATH": pmb.config.chroot_path,
@@ -70,6 +71,5 @@ def root(args, cmd, suffix="native", working_dir="/", output="log",
                   pmb.helpers.run.flat_cmd(cmd, working_dir)]
     cmd_sudo = ["sudo", "env", "-i", executables["sh"], "-c",
                 pmb.helpers.run.flat_cmd(cmd_chroot, env=env_all)]
-    kill_as_root = output in ["log", "stdout"]
     return pmb.helpers.run_core.core(args, msg, cmd_sudo, None, output,
-                                     output_return, check, kill_as_root, disable_timeout)
+                                     output_return, check, True, disable_timeout)
