@@ -1,7 +1,8 @@
-# Copyright 2020 Oliver Smith
+# Copyright 2021 Oliver Smith
 # SPDX-License-Identifier: GPL-3.0-or-later
 import multiprocessing
 import os
+import sys
 
 #
 # Exported functions
@@ -14,14 +15,18 @@ from pmb.config.merge_with_args import merge_with_args
 #
 # Exported variables (internal configuration)
 #
-version = "1.26.0"
+version = "1.29.1"
 pmb_src = os.path.normpath(os.path.realpath(__file__) + "/../../..")
 apk_keys_path = pmb_src + "/pmb/data/keys"
 
+# apk-tools minimum version
+# https://pkgs.alpinelinux.org/packages?name=apk-tools&branch=edge
 # Update this frequently to prevent a MITM attack with an outdated version
-# (which may contain a vulnerable apk/libressl, and allows an attacker to
+# (which may contain a vulnerable apk/openssl, and allows an attacker to
 # exploit the system!)
-apk_tools_static_min_version = "2.10.5-r0"
+apk_tools_min_version = {"edge": "2.12.1-r0",
+                         "v3.13": "2.12.1-r0",
+                         "v3.12": "2.10.5-r1"}
 
 # postmarketOS aports compatibility (checked against "version" in pmaports.cfg)
 pmaports_min_version = "7"
@@ -31,7 +36,10 @@ pmaports_min_version = "7"
 # see migrate_work_folder()).
 work_version = 5
 
-# Minimum required version of postmarketos-ondev (pmbootstrap install --ondev)
+# Minimum required version of postmarketos-ondev (pmbootstrap install --ondev).
+# Try to support the current versions of all channels (edge, stable). When
+# bumping > 0.4.0, remove compat code in pmb/install/_install.py (search for
+# get_ondev_pkgver).
 ondev_min_version = "0.2.0"
 
 # Programs that pmbootstrap expects to be available from the host system. Keep
@@ -50,6 +58,7 @@ config_keys = ["aports",
                "jobs",
                "kernel",
                "keymap",
+               "locale",
                "nonfree_firmware",
                "nonfree_userland",
                "ssh_keys",
@@ -59,7 +68,8 @@ config_keys = ["aports",
                "user",
                "work",
                "boot_size",
-               "extra_space"]
+               "extra_space",
+               "sudo_timer"]
 
 # Config file/commandline default values
 # $WORK gets replaced with the actual value for args.work (which may be
@@ -82,6 +92,7 @@ defaults = {
     "jobs": str(multiprocessing.cpu_count() + 1),
     "kernel": "stable",
     "keymap": "",
+    "locale": "C.UTF-8",
     "log": "$WORK/log.txt",
     "mirror_alpine": "http://dl-2.alpinelinux.org/alpine/",
     "mirrors_postmarketos": ["http://mirror.postmarketos.org/postmarketos/"],
@@ -95,9 +106,38 @@ defaults = {
     "user": "user",
     "work": os.path.expanduser("~") + "/.local/var/pmbootstrap",
     "boot_size": "128",
-    "extra_space": "0"
+    "extra_space": "0",
+    "sudo_timer": False
 }
 
+
+# Whether we're connected to a TTY (which allows things like e.g. printing
+# progress bars)
+is_interactive = sys.stdout.isatty() and \
+    sys.stderr.isatty() and \
+    sys.stdin.isatty()
+
+
+# List of available locales taken from musl-locales package; see
+# https://pkgs.alpinelinux.org/contents?name=musl-locales
+locales = [
+    "C.UTF-8",
+    "ch_DE.UTF-8",
+    "de_CH.UTF-8",
+    "de_DE.UTF-8",
+    "en_GB.UTF-8",
+    "en_US.UTF-8",
+    "es_ES.UTF-8",
+    "fr_FR.UTF-8",
+    "it_IT.UTF-8",
+    "nb_NO.UTF-8",
+    "nl_NL.UTF-8",
+    "pt_BR.UTF-8",
+    "ru_RU.UTF-8",
+    "sv_SE.UTF-8"
+]
+
+
 #
 # CHROOT
 #
@@ -263,6 +303,10 @@ apkbuild_package_attributes = {
     # removing the meta-package. (#1933). To disable this feature, use:
     # "pmbootstrap install --no-recommends".
     "_pmb_recommends": {"array": True},
+
+    # UI meta-packages can specify groups to which the user must be added
+    # to access specific hardware such as LED indicators.
+    "_pmb_groups": {"array": True},
 }
 
 # Variables in APKBUILD files, that get parsed
@@ -601,3 +645,8 @@ newapkbuild_arguments_switches_other = [
 #
 # Patterns of package names to ignore for automatic pmaport upgrading ("pmbootstrap aportupgrade --all")
 upgrade_ignore = ["device-*", "firmware-*", "linux-*", "postmarketos-*", "*-aarch64", "*-armhf", "*-armv7"]
+
+#
+# SIDELOAD
+#
+sideload_sudo_prompt = "[sudo] password for %u@%h: "
