@@ -1,4 +1,4 @@
-# Copyright 2020 Oliver Smith
+# Copyright 2021 Oliver Smith
 # SPDX-License-Identifier: GPL-3.0-or-later
 import logging
 import glob
@@ -344,7 +344,8 @@ def ask_for_additional_options(args, cfg):
                  f" extra free space: {args.extra_space} MB,"
                  f" boot partition size: {args.boot_size} MB,"
                  f" parallel jobs: {args.jobs},"
-                 f" ccache per arch: {args.ccache_size}")
+                 f" ccache per arch: {args.ccache_size},"
+                 f" sudo timer: {args.sudo_timer}")
 
     if not pmb.helpers.cli.confirm(args, "Change them?",
                                    default=False):
@@ -385,6 +386,17 @@ def ask_for_additional_options(args, cfg):
                                  lowercase_answer=False, validation_regex=regex)
     cfg["pmbootstrap"]["ccache_size"] = answer
 
+    # Sudo timer
+    logging.info("pmbootstrap does everything in Alpine Linux chroots, so"
+                 " your host system does not get modified. In order to"
+                 " work with these chroots, pmbootstrap calls 'sudo'"
+                 " internally. For long running operations, it is possible"
+                 " that you'll have to authorize sudo more than once.")
+    answer = pmb.helpers.cli.confirm(args, "Enable background timer to prevent"
+                                     " repeated sudo authorization?",
+                                     default=args.sudo_timer)
+    cfg["pmbootstrap"]["sudo_timer"] = str(answer)
+
 
 def ask_for_hostname(args, device):
     while True:
@@ -415,6 +427,16 @@ def ask_build_pkgs_on_install(args):
                                    default=args.build_pkgs_on_install)
 
 
+def ask_for_locale(args):
+    locales = pmb.config.locales
+    logging.info(f"Available locales ({len(locales)}): {', '.join(locales)}")
+    return pmb.helpers.cli.ask(args, "Choose default locale for installation",
+                               default=args.locale,
+                               lowercase_answer=False,
+                               validation_regex="|".join(locales),
+                               complete=locales)
+
+
 def frontend(args):
     require_programs()
 
@@ -471,6 +493,9 @@ def frontend(args):
     # Configure timezone info
     cfg["pmbootstrap"]["timezone"] = ask_for_timezone(args)
 
+    # Locale
+    cfg["pmbootstrap"]["locale"] = ask_for_locale(args)
+
     # Hostname
     cfg["pmbootstrap"]["hostname"] = ask_for_hostname(args, device)
 
