@@ -1,4 +1,4 @@
-# Copyright 2020 Oliver Smith
+# Copyright 2021 Oliver Smith
 # SPDX-License-Identifier: GPL-3.0-or-later
 import glob
 import json
@@ -9,6 +9,7 @@ import sys
 import pmb.aportgen
 import pmb.build
 import pmb.build.autodetect
+import pmb.sideload
 import pmb.chroot
 import pmb.chroot.initfs
 import pmb.chroot.other
@@ -69,6 +70,24 @@ def _parse_suffix(args):
         return "native"
 
 
+def _install_ondev_verify_no_rootfs(args):
+    chroot_dest = "/var/lib/rootfs.img"
+    dest = f"{args.work}/chroot_installer_{args.device}{chroot_dest}"
+    if os.path.exists(dest):
+        return
+
+    if args.ondev_cp:
+        for _, chroot_dest_cp in args.ondev_cp:
+            if chroot_dest_cp == chroot_dest:
+                return
+
+    raise ValueError(f"--no-rootfs set, but rootfs.img not found in install"
+                     " chroot. Either run 'pmbootstrap install' without"
+                     " --no-rootfs first to let it generate the postmarketOS"
+                     " rootfs once, or supply a rootfs file with:"
+                     f" --cp os.img:{chroot_dest}")
+
+
 def aportgen(args):
     for package in args.packages:
         logging.info("Generate aport: " + package)
@@ -113,6 +132,15 @@ def checksum(args):
             pmb.build.checksum.update(args, package)
 
 
+def sideload(args):
+    arch = args.deviceinfo["arch"]
+    if args.arch:
+        arch = args.arch
+    user = args.user
+    host = args.host
+    pmb.sideload.sideload(args, user, host, arch, args.install_key, args.packages)
+
+
 def chroot(args):
     # Suffix
     suffix = _parse_suffix(args)
@@ -223,6 +251,14 @@ def install(args):
         if args.rsync:
             raise ValueError("--on-device-installer cannot be combined with"
                              " --rsync")
+    else:
+        if args.ondev_cp:
+            raise ValueError("--cp can only be combined with --ondev")
+        if args.ondev_no_rootfs:
+            raise ValueError("--no-rootfs can only be combined with --ondev."
+                             " Do you mean --no-image?")
+    if args.ondev_no_rootfs:
+        _install_ondev_verify_no_rootfs(args)
 
     # On-device installer overrides
     if args.on_device_installer:
