@@ -1,10 +1,11 @@
-# Copyright 2020 Oliver Smith
+# Copyright 2021 Oliver Smith
 # SPDX-License-Identifier: GPL-3.0-or-later
 import fcntl
 import logging
 import selectors
 import subprocess
 import sys
+import threading
 import time
 import os
 import pmb.helpers.run
@@ -14,13 +15,12 @@ import pmb.helpers.run
     called by core(). """
 
 
-def sanity_checks(output="log", output_return=False, check=None,
-                  kill_as_root=False):
+def sanity_checks(output="log", output_return=False, check=None):
     """
     Raise an exception if the parameters passed to core() don't make sense
     (all parameters are described in core() below).
     """
-    if output not in ["log", "stdout", "interactive", "tui", "background"]:
+    if output not in ["log", "stdout", "interactive", "tui", "background", "pipe"]:
         raise RuntimeError("Invalid output value: " + str(output))
 
     # Prevent setting the check parameter with output="background".
@@ -33,15 +33,20 @@ def sanity_checks(output="log", output_return=False, check=None,
     if output_return and output in ["tui", "background"]:
         raise RuntimeError("Can't use output_return with output: " + output)
 
-    if kill_as_root and output in ["interactive", "tui", "background"]:
-        raise RuntimeError("Can't use kill_as_root with output: " + output)
-
 
 def background(args, cmd, working_dir=None):
     """ Run a subprocess in background and redirect its output to the log. """
     ret = subprocess.Popen(cmd, stdout=args.logfd, stderr=args.logfd,
                            cwd=working_dir)
-    logging.debug("Started process in background with PID " + str(ret.pid))
+    logging.debug(f"New background process: pid={ret.pid}, output=background")
+    return ret
+
+
+def pipe(args, cmd, working_dir=None):
+    """ Run a subprocess in background and redirect its output to a pipe. """
+    ret = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=args.logfd,
+                           cwd=working_dir)
+    logging.verbose(f"New background process: pid={ret.pid}, output=pipe")
     return ret
 
 
@@ -77,15 +82,15 @@ def pipe_read(args, process, output_to_stdout=False, output_return=False,
         return
 
 
-def kill_process_tree(args, pid, ppids, kill_as_root):
+def kill_process_tree(args, pid, ppids, sudo):
     """
     Recursively kill a pid and its child processes
 
     :param pid: process id that will be killed
     :param ppids: list of process id and parent process id tuples (pid, ppid)
-    :param kill_as_root: use sudo to kill the process
+    :param sudo: use sudo to kill the process
     """
-    if kill_as_root:
+    if sudo:
         pmb.helpers.run.root(args, ["kill", "-9", str(pid)],
                              check=False)
     else:
@@ -94,15 +99,15 @@ def kill_process_tree(args, pid, ppids, kill_as_root):
 
     for (child_pid, child_ppid) in ppids:
         if child_ppid == str(pid):
-            kill_process_tree(args, child_pid, ppids, kill_as_root)
+            kill_process_tree(args, child_pid, ppids, sudo)
 
 
-def kill_command(args, pid, kill_as_root):
+def kill_command(args, pid, sudo):
     """
     Kill a command process and recursively kill its child processes
 
     :param pid: process id that will be killed
-    :param kill_as_root: use sudo to kill the process
+    :param sudo: use sudo to kill the process
     """
     cmd = ["ps", "-e", "-o", "pid=,ppid=", "--noheaders"]
     ret = subprocess.run(cmd, check=True, stdout=subprocess.PIPE)
@@ -114,12 +119,12 @@ def kill_command(args, pid, kill_as_root):
             raise RuntimeError("Unexpected ps output: " + row)
         ppids.append(items)
 
-    kill_process_tree(args, pid, ppids, kill_as_root)
+    kill_process_tree(args, pid, ppids, sudo)
 
 
 def foreground_pipe(args, cmd, working_dir=None, output_to_stdout=False,
                     output_return=False, output_timeout=True,
-                    kill_as_root=False):
+                    sudo=False):
     """
     Run a subprocess in foreground with redirected output and optionally kill
     it after being silent for too long.
@@ -131,7 +136,7 @@ def foreground_pipe(args, cmd, working_dir=None, output_to_stdout=False,
     :param output_timeout: kill the process when it doesn't print any output
                            after a certain time (configured with --timeout)
                            and raise a RuntimeError exception
-    :param kill_as_root: use sudo to kill the process when it hits the timeout
+    :param sudo: use sudo to kill the process when it hits the timeout
     :returns: (code, output)
               * code: return code of the program
               * output: ""
@@ -165,7 +170,7 @@ def foreground_pipe(args, cmd, working_dir=None, output_to_stdout=False,
                              str(args.timeout) + " seconds. Killing it.")
                 logging.info("NOTE: The timeout can be increased with"
                              " 'pmbootstrap -t'.")
-                kill_command(args, process.pid, kill_as_root)
+                kill_command(args, process.pid, sudo)
                 continue
 
         # Read all currently available output
@@ -195,8 +200,51 @@ def foreground_tui(cmd, working_dir=None):
     return process.wait()
 
 
+def check_return_code(args, code, log_message):
+    """
+    Check the return code of a command.
+
+    :param code: exit code to check
+    :param log_message: simplified and more readable form of the command, e.g.
+                        "(native) % echo test" instead of the full command with
+                        entering the chroot and more escaping
+    :raises RuntimeError: when the code indicates that the command failed
+    """
+
+    if code:
+        logging.debug("^" * 70)
+        logging.info("NOTE: The failed command's output is above the ^^^ line"
+                     " in the log file: " + args.log)
+        raise RuntimeError("Command failed: " + log_message)
+
+
+def sudo_timer_iterate():
+    """
+    Run sudo -v and schedule a new timer to repeat the same.
+    """
+
+    subprocess.Popen(["sudo", "-v"]).wait()
+
+    timer = threading.Timer(interval=60, function=sudo_timer_iterate)
+    timer.daemon = True
+    timer.start()
+
+
+def sudo_timer_start(args):
+    """
+    Start a timer to call sudo -v periodically, so that the password is only
+    needed once.
+    """
+
+    if "sudo_timer_active" in args.cache:
+        return
+    args.cache["sudo_timer_active"] = True
+
+    sudo_timer_iterate()
+
+
 def core(args, log_message, cmd, working_dir=None, output="log",
-         output_return=False, check=None, kill_as_root=False, disable_timeout=False):
+         output_return=False, check=None, sudo=False, disable_timeout=False):
     """
     Run a command and create a log entry.
 
@@ -215,14 +263,16 @@ def core(args, log_message, cmd, working_dir=None, output="log",
                    "stdout", "interactive", "background"), so it's easy to
                    trace what pmbootstrap does.
 
-                   The exception is "tui" (text-based user interface), where
+                   The exceptions are "tui" (text-based user interface), where
                    it does not make sense to write to the log file (think of
-                   ncurses UIs, such as "menuconfig").
+                   ncurses UIs, such as "menuconfig") and "pipe" where the
+                   output is written to a pipe for manual asynchronous
+                   consumption by the caller.
 
-                   When the output is not set to "interactive", "tui" or
-                   "background", we kill the process if it does not output
-                   anything for 5 minutes (time can be set with "pmbootstrap
-                   --timeout").
+                   When the output is not set to "interactive", "tui",
+                   "background" or "pipe", we kill the process if it does not
+                   output anything for 5 minutes (time can be set with
+                   "pmbootstrap --timeout").
 
                    The table below shows all possible values along with
                    their properties. "wait" indicates that we wait for the
@@ -235,21 +285,26 @@ def core(args, log_message, cmd, working_dir=None, output="log",
                    "interactive" |         | x          | x             | x
                    "tui"         |         |            | x             | x
                    "background"  |         | x          |               |
+                   "pipe"        |         |            |               |
 
     :param output_return: in addition to writing the program's output to the
                           destinations above in real time, write to a buffer
                           and return it as string when the command has
                           completed. This is not possible when output is
-                          "background" or "tui".
+                          "background", "pipe" or "tui".
     :param check: an exception will be raised when the command's return code
                   is not 0. Set this to False to disable the check. This
-                  parameter can not be used when the output is "background".
-    :param kill_as_root: use sudo to kill the process when it hits the timeout.
+                  parameter can not be used when the output is "background" or
+                  "pipe".
+    :param sudo: use sudo to kill the process when it hits the timeout.
     :returns: * program's return code (default)
-              * subprocess.Popen instance (output is "background")
+              * subprocess.Popen instance (output is "background" or "pipe")
               * the program's entire output (output_return is True)
     """
-    sanity_checks(output, output_return, check, kill_as_root)
+    sanity_checks(output, output_return, check)
+
+    if args.sudo_timer and sudo:
+        sudo_timer_start(args)
 
     # Log simplified and full command (pmbootstrap -v)
     logging.debug(log_message)
@@ -259,6 +314,10 @@ def core(args, log_message, cmd, working_dir=None, output="log",
     if output == "background":
         return background(args, cmd, working_dir)
 
+    # Pipe
+    if output == "pipe":
+        return pipe(args, cmd, working_dir)
+
     # Foreground
     output_after_run = ""
     if output == "tui":
@@ -271,18 +330,16 @@ def core(args, log_message, cmd, working_dir=None, output="log",
             output_to_stdout = True
 
         output_timeout = output in ["log", "stdout"] and not disable_timeout
+
         (code, output_after_run) = foreground_pipe(args, cmd, working_dir,
                                                    output_to_stdout,
                                                    output_return,
                                                    output_timeout,
-                                                   kill_as_root)
+                                                   sudo)
 
     # Check the return code
-    if code and check is not False:
-        logging.debug("^" * 70)
-        logging.info("NOTE: The failed command's output is above the ^^^ line"
-                     " in the log file: " + args.log)
-        raise RuntimeError("Command failed: " + log_message)
+    if check is not False:
+        check_return_code(args, code, log_message)
 
     # Return (code or output string)
     return output_after_run if output_return else code
