@@ -1,4 +1,4 @@
-# Copyright 2020 Oliver Smith
+# Copyright 2021 Oliver Smith
 # SPDX-License-Identifier: GPL-3.0-or-later
 import logging
 import os
@@ -15,8 +15,8 @@ import pmb.config.pmaports
 import pmb.helpers.devices
 import pmb.helpers.run
 import pmb.install.blockdevice
-import pmb.install.file
 import pmb.install.recovery
+import pmb.install.ui
 import pmb.install
 
 
@@ -101,36 +101,6 @@ def get_kernel_package(args, device):
     return ["device-" + device + "-kernel-" + args.kernel]
 
 
-def get_recommends_packages(args):
-    """ Get all packages listed in _pmb_recommends of the UI and UI-extras
-        package, unless running with pmbootstrap install --no-recommends.
-
-        :returns: list of pkgnames, e.g. ["chatty", "gnome-contacts"] """
-    ret = []
-    if not args.install_recommends or args.ui == "none":
-        return ret
-
-    # UI package
-    meta = f"postmarketos-ui-{args.ui}"
-    apkbuild = pmb.helpers.pmaports.get(args, meta)
-    recommends = apkbuild["_pmb_recommends"]
-    if recommends:
-        logging.debug(f"{meta}: install _pmb_recommends:"
-                      f" {', '.join(recommends)}")
-        ret += recommends
-
-    # UI-extras subpackage
-    meta_extras = f"{meta}-extras"
-    if args.ui_extras and meta_extras in apkbuild["subpackages"]:
-        recommends = apkbuild["subpackages"][meta_extras]["_pmb_recommends"]
-        if recommends:
-            logging.debug(f"{meta_extras}: install _pmb_recommends:"
-                          f" {', '.join(recommends)}")
-            ret += recommends
-
-    return ret
-
-
 def copy_files_from_chroot(args, suffix):
     """
     Copy all files from the rootfs chroot to /mnt/install, except
@@ -150,6 +120,11 @@ def copy_files_from_chroot(args, suffix):
     if os.path.exists(qemu_binary):
         pmb.helpers.run.root(args, ["rm", qemu_binary])
 
+    # Remove apk progress fifo
+    fifo = f"{args.work}/chroot_{suffix}/tmp/apk_progress_fifo"
+    if os.path.exists(fifo):
+        pmb.helpers.run.root(args, ["rm", fifo])
+
     # Get all folders inside the device rootfs (except for home)
     folders = []
     for path in glob.glob(mountpoint_outside + "/*"):
@@ -220,10 +195,11 @@ def set_user(args):
     if not pmb.chroot.user_exists(args, args.user, suffix):
         pmb.chroot.root(args, ["adduser", "-D", "-u", "10000", args.user],
                         suffix)
-        for group in pmb.config.install_user_groups:
-            pmb.chroot.root(args, ["addgroup", "-S", group], suffix,
-                            check=False)
-            pmb.chroot.root(args, ["addgroup", args.user, group], suffix)
+    groups = pmb.install.ui.get_groups(args) + pmb.config.install_user_groups
+    for group in groups:
+        pmb.chroot.root(args, ["addgroup", "-S", group], suffix,
+                        check=False)
+        pmb.chroot.root(args, ["addgroup", args.user, group], suffix)
 
 
 def setup_login(args):
@@ -456,10 +432,14 @@ def sanity_check_sdcard_size(args):
             raise RuntimeError("Aborted.")
 
 
-def sanity_check_ondev_version(args):
+def get_ondev_pkgver(args):
     arch = args.deviceinfo["arch"]
     package = pmb.helpers.package.get(args, "postmarketos-ondev", arch)
-    ver_pkg = package["version"].split("-r")[0]
+    return package["version"].split("-r")[0]
+
+
+def sanity_check_ondev_version(args):
+    ver_pkg = get_ondev_pkgver(args)
     ver_min = pmb.config.ondev_min_version
     if pmb.parse.version.compare(ver_pkg, ver_min) == -1:
         raise RuntimeError("This version of pmbootstrap requires"
@@ -469,13 +449,15 @@ def sanity_check_ondev_version(args):
 
 
 def install_system_image(args, size_reserve, suffix, step, steps,
-                         root_label="pmOS_root", split=False, sdcard=None):
+                         boot_label="pmOS_boot", root_label="pmOS_root",
+                         split=False, sdcard=None):
     """
     :param size_reserve: empty partition between root and boot in MiB (pma#463)
     :param suffix: the chroot suffix, where the rootfs that will be installed
                    on the device has been created (e.g. "rootfs_qemu-amd64")
     :param step: next installation step
     :param steps: total installation steps
+    :param boot_label: label of the boot partition (e.g. "pmOS_boot")
     :param root_label: label of the root partition (e.g. "pmOS_root")
     :param split: create separate images for boot and root partitions
     :param sdcard: path to sdcard device (e.g. /dev/mmcblk0) or None
@@ -493,7 +475,7 @@ def install_system_image(args, size_reserve, suffix, step, steps,
         root_id = 3 if size_reserve else 2
         pmb.install.partitions_mount(args, root_id, sdcard)
 
-    pmb.install.format(args, size_reserve, root_label, sdcard)
+    pmb.install.format(args, size_reserve, boot_label, root_label, sdcard)
 
     # Just copy all the files
     logging.info(f"*** ({step + 1}/{steps}) FILL INSTALL BLOCKDEVICE ***")
@@ -603,10 +585,11 @@ def install_recovery_zip(args, steps):
 
 def install_on_device_installer(args, step, steps):
     # Generate the rootfs image
-    suffix_rootfs = f"rootfs_{args.device}"
-    install_system_image(args, 0, suffix_rootfs, step=step, steps=steps,
-                         split=True)
-    step += 2
+    if not args.ondev_no_rootfs:
+        suffix_rootfs = f"rootfs_{args.device}"
+        install_system_image(args, 0, suffix_rootfs, step=step, steps=steps,
+                             split=True)
+        step += 2
 
     # Prepare the installer chroot
     logging.info(f"*** ({step}/{steps}) CREATE ON-DEVICE INSTALLER ROOTFS ***")
@@ -619,12 +602,13 @@ def install_on_device_installer(args, step, steps):
     pmb.chroot.apk.install(args, packages, suffix_installer)
 
     # Move rootfs image into installer chroot
-    img = f"{args.device}-root.img"
-    img_path_src = f"{args.work}/chroot_native/home/pmos/rootfs/{img}"
     img_path_dest = f"{args.work}/chroot_{suffix_installer}/var/lib/rootfs.img"
-    logging.info(f"({suffix_installer}) add {img} as /var/lib/rootfs.img")
-    pmb.install.losetup.umount(args, img_path_src)
-    pmb.helpers.run.root(args, ["mv", img_path_src, img_path_dest])
+    if not args.ondev_no_rootfs:
+        img = f"{args.device}-root.img"
+        img_path_src = f"{args.work}/chroot_native/home/pmos/rootfs/{img}"
+        logging.info(f"({suffix_installer}) add {img} as /var/lib/rootfs.img")
+        pmb.install.losetup.umount(args, img_path_src)
+        pmb.helpers.run.root(args, ["mv", img_path_src, img_path_dest])
 
     # Run ondev-prepare, so it may generate nice configs from the channel
     # properties (e.g. to display the version number), or transform the image
@@ -643,16 +627,95 @@ def install_on_device_installer(args, step, steps):
            "ONDEV_UI": args.ui}
     pmb.chroot.root(args, ["ondev-prepare"], suffix_installer, env=env)
 
+    # Copy files specified with 'pmbootstrap install --ondev --cp'
+    if args.ondev_cp:
+        for host_src, chroot_dest in args.ondev_cp:
+            host_dest = f"{args.work}/chroot_{suffix_installer}/{chroot_dest}"
+            logging.info(f"({suffix_installer}) add {host_src} as"
+                         f" {chroot_dest}")
+            pmb.helpers.run.root(args, ["install", "-Dm644", host_src,
+                                        host_dest])
+
     # Remove $DEVICE-boot.img (we will generate a new one if --split was
     # specified, otherwise the separate boot image is not needed)
-    img_boot = f"{args.device}-boot.img"
-    logging.info(f"(native) rm {img_boot}")
-    pmb.chroot.root(args, ["rm", f"/home/pmos/rootfs/{img_boot}"])
+    if not args.ondev_no_rootfs:
+        img_boot = f"{args.device}-boot.img"
+        logging.info(f"(native) rm {img_boot}")
+        pmb.chroot.root(args, ["rm", f"/home/pmos/rootfs/{img_boot}"])
 
     # Generate installer image
     size_reserve = round(os.path.getsize(img_path_dest) / 1024 / 1024) + 200
+    boot_label = "pmOS_inst_boot"
+    if pmb.parse.version.compare(get_ondev_pkgver(args), "0.4.0") == -1:
+        boot_label = "pmOS_boot"
     install_system_image(args, size_reserve, suffix_installer, step, steps,
-                         "pmOS_install", args.split, args.sdcard)
+                         boot_label, "pmOS_install", args.split, args.sdcard)
+
+
+def create_device_rootfs(args, step, steps):
+    # List all packages to be installed (including the ones specified by --add)
+    # and upgrade the installed packages/apkindexes
+    logging.info(f'*** ({step}/{steps}) CREATE DEVICE ROOTFS ("{args.device}")'
+                 ' ***')
+
+    suffix = f"rootfs_{args.device}"
+    # Create user before installing packages, so post-install scripts of
+    # pmaports can figure out the username (legacy reasons: pmaports#820)
+    set_user(args)
+
+    # Fill install_packages
+    install_packages = (pmb.config.install_device_packages +
+                        ["device-" + args.device] +
+                        get_kernel_package(args, args.device) +
+                        get_nonfree_packages(args, args.device) +
+                        pmb.install.ui.get_recommends(args))
+    if not args.install_base:
+        install_packages = [p for p in install_packages if p != "postmarketos-base"]
+    if args.ui.lower() != "none":
+        install_packages += ["postmarketos-ui-" + args.ui]
+        if args.ui_extras:
+            install_packages += ["postmarketos-ui-" + args.ui + "-extras"]
+    if args.extra_packages.lower() != "none":
+        install_packages += args.extra_packages.split(",")
+    if args.add:
+        install_packages += args.add.split(",")
+    locale_is_set = (args.locale != pmb.config.defaults["locale"])
+    if locale_is_set:
+        install_packages += ["lang", "musl-locales"]
+
+    pmb.helpers.repo.update(args, args.deviceinfo["arch"])
+
+    # Explicitly call build on the install packages, to re-build them or any
+    # dependency, in case the version increased
+    if args.build_pkgs_on_install:
+        for pkgname in install_packages:
+            pmb.build.package(args, pkgname, args.deviceinfo["arch"])
+
+    # Install all packages to device rootfs chroot (and rebuild the initramfs,
+    # because that doesn't always happen automatically yet, e.g. when the user
+    # installed a hook without pmbootstrap - see #69 for more info)
+    suffix = "rootfs_" + args.device
+    pmb.chroot.apk.install(args, install_packages, suffix)
+    for flavor in pmb.chroot.other.kernel_flavors_installed(args, suffix):
+        pmb.chroot.initfs.build(args, flavor, suffix)
+
+    # Set the user password
+    setup_login(args)
+
+    # Set the keymap if the device requires it
+    setup_keymap(args)
+
+    # Set timezone
+    pmb.chroot.root(args, ["setup-timezone", "-z", args.timezone], suffix)
+
+    # Set locale
+    if locale_is_set:
+        pmb.chroot.root(args, ["sed", "-i",
+                               f"s/LANG=C.UTF-8/LANG={args.locale}/",
+                               "/etc/profile.d/locale.sh"], suffix)
+
+    # Set the hostname as the device name
+    setup_hostname(args)
 
 
 def install(args):
@@ -669,66 +732,20 @@ def install(args):
     elif args.android_recovery_zip:
         steps = 3
     elif args.on_device_installer:
-        steps = 7
+        steps = 4 if args.ondev_no_rootfs else 7
     else:
         steps = 4
 
     # Install required programs in native chroot
-    logging.info(f"*** (1/{steps}) PREPARE NATIVE CHROOT ***")
+    step = 1
+    logging.info(f"*** ({step}/{steps}) PREPARE NATIVE CHROOT ***")
     pmb.chroot.apk.install(args, pmb.config.install_native_packages,
                            build=False)
+    step += 1
 
-    logging.info(f'*** (2/{steps}) CREATE DEVICE ROOTFS ("{args.device}") ***')
-
-    # Create user before installing packages, so post-install scripts of
-    # pmaports can figure out the username (legacy reasons: pmaports#820)
-    set_user(args)
-
-    # Fill install_packages
-    install_packages = (pmb.config.install_device_packages +
-                        ["device-" + args.device] +
-                        get_kernel_package(args, args.device) +
-                        get_nonfree_packages(args, args.device) +
-                        get_recommends_packages(args))
-    if not args.install_base:
-        install_packages = [p for p in install_packages if p != "postmarketos-base"]
-    if args.ui.lower() != "none":
-        install_packages += ["postmarketos-ui-" + args.ui]
-        if args.ui_extras:
-            install_packages += ["postmarketos-ui-" + args.ui + "-extras"]
-    if args.extra_packages.lower() != "none":
-        install_packages += args.extra_packages.split(",")
-    if args.add:
-        install_packages += args.add.split(",")
-
-    pmb.helpers.repo.update(args, args.deviceinfo["arch"])
-
-    # Explicitly call build on the install packages, to re-build them or any
-    # dependency, in case the version increased
-    if args.build_pkgs_on_install:
-        for pkgname in install_packages:
-            pmb.build.package(args, pkgname, args.deviceinfo["arch"])
-
-    # Install all packages to device rootfs chroot (and rebuild the initramfs,
-    # because that doesn't always happen automatically yet, e.g. when the user
-    # installed a hook without pmbootstrap - see #69 for more info)
-    suffix = "rootfs_" + args.device
-    pmb.chroot.apk.install(args, install_packages, suffix)
-    pmb.install.file.write_os_release(args, suffix)
-    for flavor in pmb.chroot.other.kernel_flavors_installed(args, suffix):
-        pmb.chroot.initfs.build(args, flavor, suffix)
-
-    # Set the user password
-    setup_login(args)
-
-    # Set the keymap if the device requires it
-    setup_keymap(args)
-
-    # Set timezone
-    pmb.chroot.root(args, ["setup-timezone", "-z", args.timezone], suffix)
-
-    # Set the hostname as the device name
-    setup_hostname(args)
+    if not args.ondev_no_rootfs:
+        create_device_rootfs(args, step, steps)
+        step += 1
 
     if args.no_image:
         return
@@ -737,8 +754,8 @@ def install(args):
 
     if args.on_device_installer:
         # Runs install_system_image twice
-        install_on_device_installer(args, 3, steps)
+        install_on_device_installer(args, step, steps)
     else:
-        install_system_image(args, 0, f"rootfs_{args.device}", 3, steps,
+        install_system_image(args, 0, f"rootfs_{args.device}", step, steps,
                              split=args.split, sdcard=args.sdcard)
     print_flash_info(args)
