@@ -1,7 +1,8 @@
-# Copyright 2020 Oliver Smith
+# Copyright 2021 Oliver Smith
 # SPDX-License-Identifier: GPL-3.0-or-later
 import argparse
 import copy
+import os
 
 try:
     import argcomplete
@@ -22,6 +23,29 @@ import pmb.helpers.pmaports
     See pmb/helpers/args.py for more information about the args variable. """
 
 
+def type_ondev_cp(val):
+    """ Parse and validate arguments to 'pmbootstrap install --ondev --cp'.
+
+        :param val: 'HOST_SRC:CHROOT_DEST' string
+        :returns: (HOST_SRC, CHROOT_DEST) """
+    ret = val.split(":")
+
+    if len(ret) != 2:
+        raise argparse.ArgumentTypeError("does not have HOST_SRC:CHROOT_DEST"
+                                         f" format: {val}")
+    host_src = ret[0]
+    if not os.path.exists(host_src):
+        raise argparse.ArgumentTypeError(f"HOST_SRC not found: {host_src}")
+    if not os.path.isfile(host_src):
+        raise argparse.ArgumentTypeError(f"HOST_SRC is not a file: {host_src}")
+
+    chroot_dest = ret[1]
+    if not chroot_dest.startswith("/"):
+        raise argparse.ArgumentTypeError("CHROOT_DEST must start with '/':"
+                                         f" {chroot_dest}")
+    return ret
+
+
 def arguments_install(subparser):
     ret = subparser.add_parser("install", help="set up device specific"
                                " chroot and install to SD card or image file")
@@ -116,6 +140,16 @@ def arguments_install(subparser):
     group.add_argument("--no-local-pkgs", dest="install_local_pkgs",
                        help="do not install locally compiled packages and"
                             " package signing keys", action="store_false")
+    group.add_argument("--cp", dest="ondev_cp", nargs="+",
+                       metavar="HOST_SRC:CHROOT_DEST", type=type_ondev_cp,
+                       help="copy one or more files from the host system path"
+                            " HOST_SRC to the target path CHROOT_DEST")
+    group.add_argument("--no-rootfs", dest="ondev_no_rootfs",
+                       help="do not generate a pmOS rootfs as"
+                            " /var/lib/rootfs.img (install chroot). The file"
+                            " must either exist from a previous"
+                            " 'pmbootstrap install' run or by providing it"
+                            " as CHROOT_DEST with --cp", action="store_true")
 
 
 def arguments_export(subparser):
@@ -135,6 +169,23 @@ def arguments_export(subparser):
     return ret
 
 
+def arguments_sideload(subparser):
+    ret = subparser.add_parser("sideload", help="Push packages to a running"
+                               " phone connected over usb or wifi")
+    add_packages_arg(ret, nargs="+")
+    ret.add_argument("--host", help="ip of the device over wifi"
+                                    " (defaults to 172.16.42.1)",
+                     default="172.16.42.1")
+    ret.add_argument("--user", help="use a different username than the"
+                     " one set in init")
+    ret.add_argument("--arch", help="use a different architecture than the one"
+                                    " set in init")
+    ret.add_argument("--install-key", help="install the apk key from this"
+                     " machine if needed",
+                     action="store_true", dest="install_key")
+    return ret
+
+
 def arguments_flasher(subparser):
     ret = subparser.add_parser("flasher", help="flash something to the"
                                " target device")
@@ -225,6 +276,11 @@ def arguments_qemu(subparser):
     ret.add_argument("--cmdline", help="override kernel commandline")
     ret.add_argument("--image-size", default="4G",
                      help="set rootfs size, e.g. 2048M or 2G (default: 4G)")
+    ret.add_argument("--second-storage", metavar="IMAGE_SIZE",
+                     help="add a second storage with the given size (default:"
+                          " 4G), gets created if it does not exist. Use to"
+                          " test install from SD to eMMC",
+                     nargs="?", default=None, const="4G")
     ret.add_argument("-m", "--memory", type=int, default=1024,
                      help="guest RAM (default: 1024)")
     ret.add_argument("-p", "--port", type=int, default=2222,
@@ -533,6 +589,7 @@ def arguments():
     arguments_repo_missing(sub)
     arguments_kconfig(sub)
     arguments_export(sub)
+    arguments_sideload(sub)
     arguments_flasher(sub)
     arguments_initfs(sub)
     arguments_qemu(sub)
